"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeAddMarginAccountInfoInstruction = exports.makePartialLiquidateInstruction = exports.makeForceCancelOrdersInstruction = exports.makeSettleBorrowInstruction = exports.makeWithdrawInstruction = exports.makeBorrowInstruction = exports.makeSettleFundsInstruction = exports.makeCancelOrderInstruction = void 0;
const bn_js_1 = __importDefault(require("bn.js"));
const web3_js_1 = require("@solana/web3.js");
const layout_1 = require("./layout");
const token_instructions_1 = require("@project-serum/serum/lib/token-instructions");
// export function makeInitMangoGroupInstruction(
//   programId: PublicKey,
//   mangoGroupPk: PublicKey,
//   signerKey: PublicKey,
//   dexProgramId: PublicKey,
//   srmVaultPk: PublicKey,
//   mintPks: PublicKey[],
//   vaultPks: PublicKey[],
//   spotMarketPks: PublicKey[],
//   oraclePks: PublicKey[],
//   signerNonce: BN,
//   maintCollRatio: number,
//   initCollRatio: number
// ): TransactionInstruction {
//
//   new BN()
//   const keys = [
//     { isSigner: false, isWritable: true, pubkey: mangoGroupPk},
//     { isSigner: true, isWritable: false,  pubkey: ownerPk },
//     { isSigner: false,  isWritable: true, pubkey: marginAccountPk },
//     { isSigner: false, isWritable: false, pubkey: SYSVAR_CLOCK_PUBKEY },
//     { isSigner: false, isWritable: false, pubkey: dexProgramId },
//     { isSigner: false, isWritable: true, pubkey: spotMarketPk },
//     { isSigner: false, isWritable: true, pubkey: bidsPk },
//     { isSigner: false, isWritable: true, pubkey: asksPk },
//     { isSigner: false, isWritable: true, pubkey: openOrdersPk },
//     { isSigner: false, isWritable: false, pubkey: signerKey },
//     { isSigner: false, isWritable: true, pubkey: eventQueuePk },
//   ]
//
//
// }
function makeCancelOrderInstruction(programId, mangoGroupPk, ownerPk, marginAccountPk, dexProgramId, spotMarketPk, bidsPk, asksPk, openOrdersPk, signerKey, eventQueuePk, order) {
    const keys = [
        { isSigner: false, isWritable: true, pubkey: mangoGroupPk },
        { isSigner: true, isWritable: false, pubkey: ownerPk },
        { isSigner: false, isWritable: true, pubkey: marginAccountPk },
        { isSigner: false, isWritable: false, pubkey: web3_js_1.SYSVAR_CLOCK_PUBKEY },
        { isSigner: false, isWritable: false, pubkey: dexProgramId },
        { isSigner: false, isWritable: true, pubkey: spotMarketPk },
        { isSigner: false, isWritable: true, pubkey: bidsPk },
        { isSigner: false, isWritable: true, pubkey: asksPk },
        { isSigner: false, isWritable: true, pubkey: openOrdersPk },
        { isSigner: false, isWritable: false, pubkey: signerKey },
        { isSigner: false, isWritable: true, pubkey: eventQueuePk },
    ];
    const data = layout_1.encodeMangoInstruction({
        CancelOrder: {
            side: order.side,
            orderId: order.orderId,
        },
    });
    return new web3_js_1.TransactionInstruction({ keys, data, programId });
}
exports.makeCancelOrderInstruction = makeCancelOrderInstruction;
function makeSettleFundsInstruction(programId, mangoGroupPk, ownerPk, marginAccountPk, dexProgramId, spotMarketPk, openOrdersPk, signerKey, spotMarketBaseVaultPk, spotMarketQuoteVaultPk, mangoBaseVaultPk, mangoQuoteVaultPk, dexSignerKey) {
    const keys = [
        { isSigner: false, isWritable: true, pubkey: mangoGroupPk },
        { isSigner: true, isWritable: false, pubkey: ownerPk },
        { isSigner: false, isWritable: true, pubkey: marginAccountPk },
        { isSigner: false, isWritable: false, pubkey: web3_js_1.SYSVAR_CLOCK_PUBKEY },
        { isSigner: false, isWritable: false, pubkey: dexProgramId },
        { isSigner: false, isWritable: true, pubkey: spotMarketPk },
        { isSigner: false, isWritable: true, pubkey: openOrdersPk },
        { isSigner: false, isWritable: false, pubkey: signerKey },
        { isSigner: false, isWritable: true, pubkey: spotMarketBaseVaultPk },
        { isSigner: false, isWritable: true, pubkey: spotMarketQuoteVaultPk },
        { isSigner: false, isWritable: true, pubkey: mangoBaseVaultPk },
        { isSigner: false, isWritable: true, pubkey: mangoQuoteVaultPk },
        { isSigner: false, isWritable: false, pubkey: dexSignerKey },
        { isSigner: false, isWritable: false, pubkey: token_instructions_1.TOKEN_PROGRAM_ID },
    ];
    const data = layout_1.encodeMangoInstruction({ SettleFunds: {} });
    return new web3_js_1.TransactionInstruction({ keys, data, programId });
}
exports.makeSettleFundsInstruction = makeSettleFundsInstruction;
function makeBorrowInstruction(programId, mangoGroupPk, marginAccountPk, walletPk, tokenIndex, openOrders, oracles, nativeQuantity) {
    const borrowKeys = [
        { isSigner: false, isWritable: true, pubkey: mangoGroupPk },
        { isSigner: false, isWritable: true, pubkey: marginAccountPk },
        { isSigner: true, isWritable: false, pubkey: walletPk },
        { isSigner: false, isWritable: false, pubkey: web3_js_1.SYSVAR_CLOCK_PUBKEY },
        ...openOrders.map((pubkey) => ({
            isSigner: false,
            isWritable: false,
            pubkey,
        })),
        ...oracles.map((pubkey) => ({
            isSigner: false,
            isWritable: false,
            pubkey,
        })),
    ];
    const borrowData = layout_1.encodeMangoInstruction({
        Borrow: { tokenIndex: new bn_js_1.default(tokenIndex), quantity: nativeQuantity },
    });
    return new web3_js_1.TransactionInstruction({
        keys: borrowKeys,
        data: borrowData,
        programId,
    });
}
exports.makeBorrowInstruction = makeBorrowInstruction;
function makeWithdrawInstruction(programId, mangoGroupPk, marginAccountPk, walletPk, signerKey, tokenAccPk, vaultPk, openOrders, oracles, nativeQuantity) {
    const withdrawKeys = [
        { isSigner: false, isWritable: true, pubkey: mangoGroupPk },
        { isSigner: false, isWritable: true, pubkey: marginAccountPk },
        { isSigner: true, isWritable: false, pubkey: walletPk },
        { isSigner: false, isWritable: true, pubkey: tokenAccPk },
        { isSigner: false, isWritable: true, pubkey: vaultPk },
        { isSigner: false, isWritable: false, pubkey: signerKey },
        { isSigner: false, isWritable: false, pubkey: token_instructions_1.TOKEN_PROGRAM_ID },
        { isSigner: false, isWritable: false, pubkey: web3_js_1.SYSVAR_CLOCK_PUBKEY },
        ...openOrders.map((pubkey) => ({
            isSigner: false,
            isWritable: false,
            pubkey,
        })),
        ...oracles.map((pubkey) => ({
            isSigner: false,
            isWritable: false,
            pubkey,
        })),
    ];
    const withdrawData = layout_1.encodeMangoInstruction({
        Withdraw: { quantity: nativeQuantity },
    });
    return new web3_js_1.TransactionInstruction({
        keys: withdrawKeys,
        data: withdrawData,
        programId,
    });
}
exports.makeWithdrawInstruction = makeWithdrawInstruction;
function makeSettleBorrowInstruction(programId, mangoGroupPk, marginAccountPk, walletPk, tokenIndex, nativeQuantity) {
    const keys = [
        { isSigner: false, isWritable: true, pubkey: mangoGroupPk },
        { isSigner: false, isWritable: true, pubkey: marginAccountPk },
        { isSigner: true, isWritable: false, pubkey: walletPk },
        { isSigner: false, isWritable: false, pubkey: web3_js_1.SYSVAR_CLOCK_PUBKEY },
    ];
    const data = layout_1.encodeMangoInstruction({
        SettleBorrow: { tokenIndex: new bn_js_1.default(tokenIndex), quantity: nativeQuantity },
    });
    return new web3_js_1.TransactionInstruction({ keys, data, programId });
}
exports.makeSettleBorrowInstruction = makeSettleBorrowInstruction;
function makeForceCancelOrdersInstruction(programId, mangoGroup, liqor, liqeeMarginAccount, baseVault, quoteVault, spotMarket, bids, asks, signerKey, dexEventQueue, dexBaseVault, dexQuoteVault, dexSigner, dexProgramId, openOrders, oracles, limit) {
    const keys = [
        { isSigner: false, isWritable: true, pubkey: mangoGroup },
        { isSigner: true, isWritable: false, pubkey: liqor },
        { isSigner: false, isWritable: true, pubkey: liqeeMarginAccount },
        { isSigner: false, isWritable: true, pubkey: baseVault },
        { isSigner: false, isWritable: true, pubkey: quoteVault },
        { isSigner: false, isWritable: true, pubkey: spotMarket },
        { isSigner: false, isWritable: true, pubkey: bids },
        { isSigner: false, isWritable: true, pubkey: asks },
        { isSigner: false, isWritable: false, pubkey: signerKey },
        { isSigner: false, isWritable: true, pubkey: dexEventQueue },
        { isSigner: false, isWritable: true, pubkey: dexBaseVault },
        { isSigner: false, isWritable: true, pubkey: dexQuoteVault },
        { isSigner: false, isWritable: false, pubkey: dexSigner },
        { isSigner: false, isWritable: false, pubkey: token_instructions_1.TOKEN_PROGRAM_ID },
        { isSigner: false, isWritable: false, pubkey: dexProgramId },
        { isSigner: false, isWritable: false, pubkey: web3_js_1.SYSVAR_CLOCK_PUBKEY },
        ...openOrders.map((pubkey) => ({
            isSigner: false,
            isWritable: true,
            pubkey,
        })),
        ...oracles.map((pubkey) => ({
            isSigner: false,
            isWritable: false,
            pubkey,
        })),
    ];
    const data = layout_1.encodeMangoInstruction({ ForceCancelOrders: { limit } });
    return new web3_js_1.TransactionInstruction({ keys, data, programId });
}
exports.makeForceCancelOrdersInstruction = makeForceCancelOrdersInstruction;
function makePartialLiquidateInstruction(programId, mangoGroup, liqor, liqorInTokenWallet, liqorOutTokenWallet, liqeeMarginAccount, inTokenVault, outTokenVault, signerKey, openOrders, oracles, maxDeposit) {
    const keys = [
        { isSigner: false, isWritable: true, pubkey: mangoGroup },
        { isSigner: true, isWritable: false, pubkey: liqor },
        { isSigner: false, isWritable: true, pubkey: liqorInTokenWallet },
        { isSigner: false, isWritable: true, pubkey: liqorOutTokenWallet },
        { isSigner: false, isWritable: true, pubkey: liqeeMarginAccount },
        { isSigner: false, isWritable: true, pubkey: inTokenVault },
        { isSigner: false, isWritable: true, pubkey: outTokenVault },
        { isSigner: false, isWritable: false, pubkey: signerKey },
        { isSigner: false, isWritable: false, pubkey: token_instructions_1.TOKEN_PROGRAM_ID },
        { isSigner: false, isWritable: false, pubkey: web3_js_1.SYSVAR_CLOCK_PUBKEY },
        ...openOrders.map((pubkey) => ({
            isSigner: false,
            isWritable: false,
            pubkey,
        })),
        ...oracles.map((pubkey) => ({
            isSigner: false,
            isWritable: false,
            pubkey,
        })),
    ];
    const data = layout_1.encodeMangoInstruction({ PartialLiquidate: { maxDeposit } });
    return new web3_js_1.TransactionInstruction({ keys, data, programId });
}
exports.makePartialLiquidateInstruction = makePartialLiquidateInstruction;
function makeAddMarginAccountInfoInstruction(programId, mangoGroup, marginAccount, owner, info) {
    const keys = [
        { isSigner: false, isWritable: true, pubkey: mangoGroup },
        { isSigner: false, isWritable: true, pubkey: marginAccount },
        { isSigner: true, isWritable: false, pubkey: owner },
    ];
    // TODO convert info into a 32 byte utf encoded byte array
    const encoded = Buffer.from(info);
    if (encoded.length > layout_1.INFO_LEN) {
        throw new Error("info string too long. Must be less than or equal to 32 bytes");
    }
    const infoArray = new Uint8Array(encoded, 0, layout_1.INFO_LEN);
    const data = layout_1.encodeMangoInstruction({ AddMarginAccountInfo: { info: infoArray } });
    return new web3_js_1.TransactionInstruction({ keys, data, programId });
}
exports.makeAddMarginAccountInfoInstruction = makeAddMarginAccountInfoInstruction;
//# sourceMappingURL=instruction.js.map